#include "StdAfx.h"
#include "RheaGameSession.h"


void CRheaGameSession::start()
{
	m_RheaServerWorld_.join(shared_from_this());
	boost::asio::async_read(m_socket_,
		boost::asio::buffer(m_read_msg_.data(), CDataMessage::header_length),
		boost::bind(
		&CRheaGameSession::handle_read_header, shared_from_this(),
		boost::asio::placeholders::error));
}

void CRheaGameSession::deliver(const CDataMessage& msg)
{
	bool write_in_progress = !m_write_msgs_.empty();
	m_write_msgs_.push_back(msg);
	if (!write_in_progress)
	{
		boost::asio::async_write(m_socket_,
			boost::asio::buffer(m_write_msgs_.front().data(),
			m_write_msgs_.front().length()),
			boost::bind(&CRheaGameSession::handle_write, shared_from_this(),
			boost::asio::placeholders::error));
	}
}

void CRheaGameSession::handle_read_header(const boost::system::error_code& error)
{
	if (!error && m_read_msg_.decode_header())
	{
		boost::asio::async_read(m_socket_,
			boost::asio::buffer(m_read_msg_.body(), m_read_msg_.body_length()),
			boost::bind(&CRheaGameSession::handle_read_body, shared_from_this(),
			boost::asio::placeholders::error));
	}
	else
	{
		m_RheaServerWorld_.leave(shared_from_this());
	}
}

void CRheaGameSession::handle_read_body(const boost::system::error_code& error)
{
	if (!error)
	{
		m_RheaServerWorld_.deliver(m_read_msg_);
		boost::asio::async_read(m_socket_,
			boost::asio::buffer(m_read_msg_.data(), CDataMessage::header_length),
			boost::bind(&CRheaGameSession::handle_read_header, shared_from_this(),
			boost::asio::placeholders::error));
	}
	else
	{
		m_RheaServerWorld_.leave(shared_from_this());
	}
}

void CRheaGameSession::handle_write(const boost::system::error_code& error)
{
	if (!error)
	{
		m_write_msgs_.pop_front();
		if (!m_write_msgs_.empty())
		{
			boost::asio::async_write(m_socket_,
				boost::asio::buffer(m_write_msgs_.front().data(),
				m_write_msgs_.front().length()),
				boost::bind(&CRheaGameSession::handle_write, shared_from_this(),
				boost::asio::placeholders::error));
		}
	}
	else
	{
		m_RheaServerWorld_.leave(shared_from_this());
	}
}