#pragma once
class CRheaGameSession :public CRheaServerParticipant, public boost::enable_shared_from_this<CRheaGameSession>
{
public:
	CRheaGameSession(boost::asio::io_service& io_service, CGLiveWorld& gLiveWorld)
		: socket_(io_service),
		gLiveWorld_(gLiveWorld)
	{
	}

	tcp::socket& socket()
	{
		return socket_;
	}

	void start()
	{
		gLiveWorld_.join(shared_from_this());
		boost::asio::async_read(socket_,
			boost::asio::buffer(read_msg_.data(), CDataMessage::header_length),
			boost::bind(
			&CRheaGameSession::handle_read_header, shared_from_this(),
			boost::asio::placeholders::error));
	}

	void deliver(const CDataMessage& msg)
	{
		bool write_in_progress = !write_msgs_.empty();
		write_msgs_.push_back(msg);
		if (!write_in_progress)
		{
			boost::asio::async_write(socket_,
				boost::asio::buffer(write_msgs_.front().data(),
				write_msgs_.front().length()),
				boost::bind(&CRheaGameSession::handle_write, shared_from_this(),
				boost::asio::placeholders::error));
		}
	}

	void handle_read_header(const boost::system::error_code& error)
	{
		if (!error && read_msg_.decode_header())
		{
			boost::asio::async_read(socket_,
				boost::asio::buffer(read_msg_.body(), read_msg_.body_length()),
				boost::bind(&CRheaGameSession::handle_read_body, shared_from_this(),
				boost::asio::placeholders::error));
		}
		else
		{
			gLiveWorld_.leave(shared_from_this());
		}
	}

	void handle_read_body(const boost::system::error_code& error)
	{
		if (!error)
		{
			gLiveWorld_.deliver(read_msg_);
			boost::asio::async_read(socket_,
				boost::asio::buffer(read_msg_.data(), CDataMessage::header_length),
				boost::bind(&CRheaGameSession::handle_read_header, shared_from_this(),
				boost::asio::placeholders::error));
		}
		else
		{
			gLiveWorld_.leave(shared_from_this());
		}
	}

	void handle_write(const boost::system::error_code& error)
	{
		if (!error)
		{
			write_msgs_.pop_front();
			if (!write_msgs_.empty())
			{
				boost::asio::async_write(socket_,
					boost::asio::buffer(write_msgs_.front().data(),
					write_msgs_.front().length()),
					boost::bind(&CRheaGameSession::handle_write, shared_from_this(),
					boost::asio::placeholders::error));
			}
		}
		else
		{
			gLiveWorld_.leave(shared_from_this());
		}
	}

private:
	tcp::socket m_socket_;
	CRheaServerWorld& m_gLiveWorld_;
	CDataMessage m_read_msg_;
	DataMessageQueue m_write_msgs_;
};

typedef boost::shared_ptr<CRheaGameSession> cms_session_ptr;